<?php
namespace Ep\Gath\Plugin;

use Inti\Provider\Query\QueryInterface;
use Inti\Provider\Query\QueryPaginateInterface;
use Inti\Provider\Query\QueryParentInterface;
use Inti\Provider\Query\QuerySlugInterface;
use Inti\Wp\Provider\Type\Term\TaxonomyInterface;
use WP_Term;
use WP_Term_Query;

class Category extends AbstractGathPlugin
{
    private $results = [];
    private $meta = [];

    public function attach(): void
    {
        add_filter('query', [$this, 'query']);
        add_filter('terms_pre_query', [$this, 'terms_pre_query'], 10, 2);
        add_filter('get_term_metadata', [$this, 'get_term_meta'], 10, 3);

        $this->loadAllCategories();
    }

    public function get_term_meta($meta, $object_id, $meta_key)
    {
        if ($object_id < Products::TERM_OFFSET_CATEGORY) {
            return $meta;
        }

        if (!isset($this->meta[$object_id])) {
            if (!isset($this->results[$object_id])) {
                return $meta;
            }

            $result = $this->results[$object_id];
            global $wpdb;

            $sql = "SELECT term_id, meta_key, meta_value 
                    FROM wp_termmeta 
                    WHERE meta_key IN('_sale_badge_text', '_sale_badge_size')
                    AND term_id = " . $object_id;

            $this->meta[$object_id] = [
                '_sale_badge_size' => [''],
                '_sale_badge_text' => [''],
                '_upsale_artcodes' => [$result['_upsale_artcodes'] ?? ''],
                '_upsale_tecdoc_artid' => [$result['_upsale_tecdoc_artid'] ?? ''],
                'genericArticleIds' => [$result['genericArticleIds'] ?? ''],
                'genericArticleMotoIds' => [$result['genericArticleMotoIds'] ?? ''],
                'IsTyreCategory' => [$result['IsTyreCategory']],
                'WebPutanjaSlike' => [$result['image'] ?? ''],
                'thumbnail_id' => $result['image'] ? [$this->images->getImageId($result['image'] ?? '')] : [''],
            ];

            $results = $wpdb->get_results($sql);

            foreach ($results as $result) {
                $this->meta[$object_id][$result->meta_key] = [$result->meta_value];
            }
        }

        return $meta_key ? $this->meta[$object_id][$meta_key] : $this->meta[$object_id];
    }

    public function terms_pre_query(?array $terms, WP_Term_Query $query)
    {
        $q = &$query->query_vars;
        $taxonomy = is_array($q['taxonomy']) ? $q['taxonomy'][0] : $q['taxonomy'];

        if (!is_string($taxonomy) || empty($taxonomy) || $taxonomy !== 'product_cat') {
            return $terms;
        }

        if (!empty($q['slug']) &&
            ((is_array($q['slug']) && in_array($q['slug'][0], ['nekategorizirane', 'uncategorized']))
            || is_string($q['slug']) && in_array($q['slug'], ['nekategorizirane', 'uncategorized']))
        ) {
            return [];
        }

        if ($q['parent'] !== '') {
            $parent = intval($q['parent']);
            if ($parent > 0) {
                $parent = intval($parent - Products::TERM_OFFSET_CATEGORY);
            }

            $data = array_filter($this->results, function ($result) use ($parent) {
                return $result['ParentDataLink'] === $parent;
            });
            /*$result = $this->gath->getCategories($parent, 0);
            $data = $result['Results'];*/
        } else if (!empty($q['include'])) {
            foreach ($q['include'] as $id) {
                $id = intval($id - Products::TERM_OFFSET_CATEGORY);

                if (isset($this->results[$id])) {
                    $data[] = $this->results[$id];
                }
                /*
                $result = $this->gath->getCategoryById($id);
                if (!empty($result['Result'])) {
                    $data[] = $result['Result'];
                }
                */
            }
        } else if (!empty($q['slug'])) {
            $data = array_filter($this->results, function ($result) use ($q) {
                return ((is_array($q['slug']) && $q['slug'][0] === $result['Slug']) || $q['slug'] === $result['Slug']);
            });
            /*
            $result = $this->gath->getCategoryBySlug($q['slug']);
            if (!empty($result['Result'])) {
                $data = [$result['Result']];
            }*/
        } else if (isset($q['genericArticleId']) && !empty($q['genericArticleId'])) {
            foreach ($this->results as $result) {
                $genericArticleIds = array_map('intval', explode(';', $result['genericArticleIds']));

                if (in_array($q['genericArticleId'], $genericArticleIds)) {
                    $data = [$result];
                    break;
                }
            }
        } else {
            $data = $this->results;
            //$data = $this->gath->getAllCategories();
        }

        $fields = $q['fields'] ?: 'all';

        if ($fields === 'count') {
            return count($data);
        }

        foreach ($data ?? [] as $row) {
            $row = $this->prepareRow($row);

            switch ($fields) {
                case 'id=>parent':
                    $terms[$row['term_id']] = $row['parent'];
                    break;
                case 'tt_ids':
                    $terms[] = $row['term_id'];
                    break;
                case 'all':
                case 'all_with_object_id':
                default:
                    $terms[] = new WP_Term((object) $row);
            }
        }

        return $terms ?? [];
    }

    public function query(string $query): string
    {
        if (preg_match('/^SELECT t\.\*, tt\.\* FROM wp_terms AS t INNER JOIN wp_term_taxonomy AS tt ON t\.term_id = tt\.term_id WHERE t\.term_id = (\d+)$/', $query, $matches)) {
            $term_id = intval($matches[1]);

            if ($term_id > Products::TERM_OFFSET_CATEGORY) {
                if (isset($this->results[$term_id])) {
                    $row = $this->prepareRow($this->results[$term_id]);
                } else {
                    $term_id = intval($term_id - Products::TERM_OFFSET_CATEGORY);
                    $result = $this->gath->getCategoryById($term_id);

                    if (!empty($result['Result'])) {
                        $row = $this->prepareRow($result['Result']);
                    }
                }

                if (isset($row) && !empty($row)) {
                    $query = $this->arrayToSelect($row);
                }
            }
        }

        return $query;
    }

    private function loadAllCategories(): void
    {
        $data = $this->gath->getAllCategories();

        foreach ($data as $result) {
            $id = intval(Products::TERM_OFFSET_CATEGORY + $result['DataLink']);
            $this->results[$id] = $result;
            $term = new WP_Term((object) $this->prepareRow($result));
            wp_cache_add($term->term_id, $term, 'terms');
        }
    }

    private function prepareRow(array $row): array
    {
        $id = intval(Products::TERM_OFFSET_CATEGORY + $row['DataLink']);
        $this->results[$id] = $row;

        if (isset($row['ParentDataLink']) && $row['ParentDataLink'] > 0) {
            $parent = intval(Products::TERM_OFFSET_CATEGORY + $row['ParentDataLink']);
        } else {
            $parent = 0;
        }

        return [
            'term_id' => $id,
            'name' => $row['Name'],
            'slug' => $row['Slug'],
            'term_group' => 0,
            'term_taxonomy_id' => $id,
            'taxonomy' => 'product_cat',
            'description' => $row['Description'] ?? '',
            'parent' => $parent,
            'count' => $row['ArticleCount'],
        ];
    }
}
